@@ -9,10 +9,13 @@ Modifications:
 
 '''
 
+import MySQLdb
+from _mysql_exceptions import ProgrammingError, OperationalError
 import csv
+import re
 import subprocess
 import tempfile
-import MySQLdb
+
 
 class MySQLDB(object):
     '''
@@ -22,6 +25,8 @@ class MySQLDB(object):
             print result
     '''
 
+    NON_ASCII_CHARS_PATTERN = re.compile(r'[^\x00-\x7F]+')
+
     def __init__(self, host='127.0.0.1', port=3306, user='root', passwd='', db='mysql'):
         '''
         
@@ -49,7 +54,8 @@ class MySQLDB(object):
         try:
             #self.connection = pymysql.connect(host=host, port=port, user=user, passwd=passwd, db=db)
             #self.connection = pymysql.connect(host=host, port=port, user=user, passwd=passwd, db=db,charset='utf8')             
-            self.connection = MySQLdb.connect(host=host, port=port, user=user, passwd=passwd, db=db, local_infile=1)
+            #self.connection = MySQLdb.connect(host=host, port=port, user=user, passwd=passwd, db=db, local_infile=1)
+            self.connection = MySQLdb.connect(host=host, port=port, user=user, passwd=passwd, db=db,charset='utf8')             
         
         #except MySQLdb.OperationalError:
         except MySQLdb.OperationalError:
@@ -57,6 +63,15 @@ class MySQLDB(object):
             raise ValueError('Cannot reach MySQL server with host:%s, port:%s, user:%s, pwd:%s, db:%s' %
                              (host, port, user, pwd, db))
         
+    def dbName(self):
+        '''
+        Return name of database to which this MySQLDB is connected.
+        
+        :return: name of database within MySQL server to which MySQLDB instance is connected.
+        :rtype: String
+        '''
+        return self.db
+    
     def close(self):
         '''
         Close all cursors that are currently still open.
@@ -81,6 +96,7 @@ class MySQLDB(object):
         :type tableName: String
         :param schema: dictionary mapping column names to column types
         :type schema: Dict<String,String>
+        :raise ValueError: if table to truncate does not exist or permissions issues.        
         '''
         colSpec = ''
         for colName, colVal in schema.items():
@@ -99,11 +115,16 @@ class MySQLDB(object):
 
         :param tableName: name of table
         :type tableName: String
+        :raise ValueError: if table to drop does not exist or permissions issues.        
         '''
         cursor = self.connection.cursor()
         try:
             cursor.execute('DROP TABLE IF EXISTS %s' % tableName)
             self.connection.commit()
+        except OperationalError as e:
+            raise ValueError("In pymysql_utils dropTable(): %s" % `e`)
+        except ProgrammingError as e:
+            raise ValueError("In pymysql_utils dropTable(): %s" % `e`)
         finally:
             cursor.close()
 
@@ -113,10 +134,16 @@ class MySQLDB(object):
 
         :param tableName: name of table
         :type tableName: String
+        :raise ValueError: if table to truncate does not exist or permissions issues.
         '''
         cursor = self.connection.cursor()
         try:
-            cursor.execute('TRUNCATE TABLE %s' % tableName)
+            try:
+                cursor.execute('TRUNCATE TABLE %s' % tableName)
+            except OperationalError as e:
+                raise ValueError("In pymysql_utils truncateTable(): %s." % `e`)
+            except ProgrammingError as e:
+                raise ValueError("In pymysql_utils truncateTable(): %s." % `e`)
             self.connection.commit()
         finally:
             cursor.close()
@@ -134,7 +161,8 @@ class MySQLDB(object):
         colNames, colValues = zip(*colnameValueDict.items())
         cursor = self.connection.cursor()
         try:
-            cmd = 'INSERT INTO %s (%s) VALUES (%s)' % (str(tblName), ','.join(colNames), self.ensureSQLTyping(colValues))
+            wellTypedColValues = self.ensureSQLTyping(colValues)
+            cmd = 'INSERT INTO %s (%s) VALUES (%s)' % (str(tblName), ','.join(colNames), wellTypedColValues)
             cursor.execute(cmd)
             self.connection.commit()
         finally:
@@ -238,10 +266,14 @@ class MySQLDB(object):
         Executes arbitrary query that is parameterized
         as in the Python string format statement. Ex:
         executeParameterized('SELECT %s FROM myTable', ('col1', 'col3'))
+        Note: params must be a tuple. Example: to update a column:: 
+            mysqldb.executeParameterized("UPDATE myTable SET col1=%s", (myVal,))
+        the comma after 'myVal' is mandatory; it indicates that the 
+        expression is a tuple.
 
         :param query: query with parameter placeholder
         :type query: string
-        :param params: actuals for the parameters
+        :param params: tuple of actuals for the parameters.
         :type params: (<any>)
         '''
         cursor=self.connection.cursor()                                                                        
@@ -264,10 +296,23 @@ class MySQLDB(object):
         resList = []
         for el in colVals:
             if isinstance(el, basestring):
-                resList.append('"%s"' % el.encode('UTF-8', 'ignore'))
-            else:
+                try:
+                    # If value is not already UTF-8, encode it:
+                    cleanStr = unicode(el, 'UTF-8', 'replace')
+                except TypeError:
+                    # Value was already in Unicode, so all is well:
+                    cleanStr = el
+                resList.append('"%s"' % cleanStr)
+            elif el is None:
+                resList.append('null')
+            elif isinstance(el, (list, dict, set)):
+                resList.append('"%s"' % str(el))
+            else: # e.g. numbers
                 resList.append(el)
-        return ','.join(map(str,resList))        
+        try:
+            return ','.join(map(unicode,resList))
+        except UnicodeEncodeError as e:
+            print('Unicode related error: %s' % `e`)
         
     def query(self, queryStr):
         '''
